@@ -2,7 +2,7 @@
    nslcd.c - ldap local connection daemon
 
    Copyright (C) 2006 West Consulting
-   Copyright (C) 2006, 2007 Arthur de Jong
+   Copyright (C) 2006, 2007, 2008 Arthur de Jong
 
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
@@ -25,7 +25,9 @@
 #include <stdlib.h>
 #include <stdio.h>
 #include <string.h>
+#ifdef HAVE_STDINT_H
 #include <stdint.h>
+#endif /* HAVE_STDINT_H */
 #include <sys/types.h>
 #include <sys/param.h>
 #include <sys/wait.h>
@@ -43,48 +45,52 @@
 #ifdef HAVE_GRP_H
 #include <grp.h>
 #endif /* HAVE_GRP_H */
+#ifdef HAVE_NSS_H
 #include <nss.h>
+#endif /* HAVE_NSS_H */
 #include <pthread.h>
+#ifndef HAVE_GETOPT_LONG
+#include "compat/getopt_long.h"
+#endif /* not HAVE_GETOPT_LONG */
+#ifndef HAVE_DAEMON
+#include "compat/daemon.h"
+#endif /* not HAVE_DAEMON */
 
 #include "nslcd.h"
 #include "log.h"
 #include "cfg.h"
 #include "common.h"
 #include "compat/attrs.h"
+#include "compat/getpeercred.h"
 
-
-/* the definition of the environment */
-extern char **environ;
-
+/* buffer sizes for I/O */
+#define READBUFFER_MINSIZE 32
+#define READBUFFER_MAXSIZE 64
+#define WRITEBUFFER_MINSIZE 64
+#define WRITEBUFFER_MAXSIZE 1*1024*1024
 
 /* flag to indictate if we are in debugging mode */
 static int nslcd_debugging=0;
 
-
 /* the exit flag to indicate that a signal was received */
 static volatile int nslcd_exitsignal=0;
 
-
 /* the server socket used for communication */
 static int nslcd_serversocket=-1;
 
-
 /* thread ids of all running threads */
-#define NUM_THREADS 5
-static pthread_t nslcd_threads[NUM_THREADS];
-
+static pthread_t *nslcd_threads;
 
 /* display version information */
 static void display_version(FILE *fp)
 {
   fprintf(fp,"%s\n",PACKAGE_STRING);
   fprintf(fp,"Written by Luke Howard and Arthur de Jong.\n\n");
-  fprintf(fp,"Copyright (C) 1997-2006 Luke Howard, Arthur de Jong and West Consulting\n"
+  fprintf(fp,"Copyright (C) 1997-2008 Luke Howard, Arthur de Jong and West Consulting\n"
              "This is free software; see the source for copying conditions.  There is NO\n"
              "warranty; not even for MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.\n");
 }
 
-
 /* display usage information to stdout and exit(status) */
 static void display_usage(FILE *fp,const char *program_name)
 {
@@ -97,7 +103,6 @@ static void display_usage(FILE *fp,const char *program_name)
              "Report bugs to <%s>.\n",PACKAGE_BUGREPORT);
 }
 
-
 /* the definition of options for getopt(). see getopt(2) */
 static struct option const nslcd_options[] =
 {
@@ -108,7 +113,6 @@ static struct option const nslcd_options[] =
 };
 #define NSLCD_OPTIONSTRING "dhV"
 
-
 /* parse command line options and save settings in struct  */
 static void parse_cmdline(int argc,char *argv[])
 {
@@ -146,7 +150,6 @@ static void parse_cmdline(int argc,char *argv[])
   }
 }
 
-
 /* get a name of a signal with a given signal number */
 static const char *signame(int signum)
 {
@@ -202,14 +205,13 @@ static const char *signame(int signum)
   }
 }
 
-
 /* signal handler for closing down */
 static RETSIGTYPE sigexit_handler(int signum)
 {
   int i;
   nslcd_exitsignal=signum;
   /* cancel all running threads */
-  for (i=0;i<NUM_THREADS;i++)
+  for (i=0;i<nslcd_cfg->ldc_threads;i++)
     if (pthread_cancel(nslcd_threads[i]))
     {
       /* TODO: figure out if we can actually log from within a signal handler */
@@ -217,7 +219,6 @@ static RETSIGTYPE sigexit_handler(int signum)
     }
 }
 
-
 /* do some cleaning up before terminating */
 static void exithandler(void)
 {
@@ -243,34 +244,29 @@ static void exithandler(void)
   log_log(LOG_INFO,"version %s bailing out",VERSION);
 }
 
-
 /* returns a socket ready to answer requests from the client,
    exit()s on error */
-static int open_socket(void)
+static int create_socket(void)
 {
   int sock;
   struct sockaddr_un addr;
-
   /* create a socket */
   if ( (sock=socket(PF_UNIX,SOCK_STREAM,0))<0 )
   {
     log_log(LOG_ERR,"cannot create socket: %s",strerror(errno));
     exit(EXIT_FAILURE);
   }
-
   /* remove existing named socket */
   if (unlink(NSLCD_SOCKET)<0)
   {
     log_log(LOG_DEBUG,"unlink() of "NSLCD_SOCKET" failed (ignored): %s",
             strerror(errno));
   }
-
   /* create socket address structure */
   memset(&addr,0,sizeof(struct sockaddr_un));
   addr.sun_family=AF_UNIX;
   strncpy(addr.sun_path,NSLCD_SOCKET,sizeof(addr.sun_path));
   addr.sun_path[sizeof(addr.sun_path)-1]='\0';
-
   /* bind to the named socket */
   if (bind(sock,(struct sockaddr *)&addr,sizeof(struct sockaddr_un)))
   {
@@ -280,7 +276,6 @@ static int open_socket(void)
       log_log(LOG_WARNING,"problem closing socket: %s",strerror(errno));
     exit(EXIT_FAILURE);
   }
-
   /* close the file descriptor on exit */
   if (fcntl(sock,F_SETFD,FD_CLOEXEC)<0)
   {
@@ -289,7 +284,6 @@ static int open_socket(void)
       log_log(LOG_WARNING,"problem closing socket: %s",strerror(errno));
     exit(EXIT_FAILURE);
   }
-
   /* set permissions of socket so anybody can do requests */
   /* Note: we use chmod() here instead of fchmod() because
      fchmod does not work on sockets
@@ -302,7 +296,6 @@ static int open_socket(void)
       log_log(LOG_WARNING,"problem closing socket: %s",strerror(errno));
     exit(EXIT_FAILURE);
   }
-
   /* start listening for connections */
   if (listen(sock,SOMAXCONN)<0)
   {
@@ -311,7 +304,6 @@ static int open_socket(void)
       log_log(LOG_WARNING,"problem closing socket: %s",strerror(errno));
     exit(EXIT_FAILURE);
   }
-
   /* we're done */
   return sock;
 }
@@ -338,50 +330,39 @@ static int read_header(TFILE *fp,int32_t *action)
 static void handleconnection(int sock,MYLDAP_SESSION *session)
 {
   TFILE *fp;
-  socklen_t alen;
-  struct ucred client;
   int32_t action;
   struct timeval readtimeout,writetimeout;
-
-  /* initialize client information (in case getsockopt() breaks) */
-  client.pid=(pid_t)0;
-  client.uid=(uid_t)-1;
-  client.gid=(gid_t)-1;
-
-  /* look up process information from client */
-  alen=(socklen_t)sizeof(struct ucred);
-  if (getsockopt(sock,SOL_SOCKET,SO_PEERCRED,&client,&alen) < 0)
-  {
-    log_log(LOG_ERR,"getsockopt(SO_PEERCRED) failed: %s",strerror(errno));
-    if (close(sock))
-      log_log(LOG_WARNING,"problem closing socket: %s",strerror(errno));
-    return;
-  }
-
+  uid_t uid;
+  gid_t gid;
+  pid_t pid;
+  /* indicate new connection to logging module (genrates unique id) */
+  log_newsession();
   /* log connection */
-  log_log(LOG_DEBUG,"connection from pid=%d uid=%d gid=%d",
-                    (int)client.pid,(int)client.uid,(int)client.gid);
-
+  if (getpeercred(sock,&uid,&gid,&pid))
+    log_log(LOG_DEBUG,"connection from unknown client: %s",strerror(errno));
+  else
+    log_log(LOG_DEBUG,"connection from pid=%d uid=%d gid=%d",
+                      (int)pid,(int)uid,(int)gid);
   /* set the timeouts */
-  readtimeout.tv_sec=0; /* clients should send there request quickly */
+  readtimeout.tv_sec=0; /* clients should send their request quickly */
   readtimeout.tv_usec=500000;
-  writetimeout.tv_sec=5; /* clients could be taking some time to process the results */
+  writetimeout.tv_sec=60; /* clients could be taking some time to process the results */
   writetimeout.tv_usec=0;
   /* create a stream object */
-  if ((fp=tio_fdopen(sock,&readtimeout,&writetimeout))==NULL)
+  if ((fp=tio_fdopen(sock,&readtimeout,&writetimeout,
+                     READBUFFER_MINSIZE,READBUFFER_MAXSIZE,
+                     WRITEBUFFER_MINSIZE,WRITEBUFFER_MAXSIZE))==NULL)
   {
     log_log(LOG_WARNING,"cannot create stream for writing: %s",strerror(errno));
     (void)close(sock);
     return;
   }
-
   /* read request */
   if (read_header(fp,&action))
   {
     (void)tio_close(fp);
     return;
   }
-
   /* handle request */
   switch (action)
   {
@@ -413,13 +394,12 @@ static void handleconnection(int sock,MYLDAP_SESSION *session)
     case NSLCD_ACTION_SERVICE_BYNAME:   (void)nslcd_service_byname(fp,session); break;
     case NSLCD_ACTION_SERVICE_BYNUMBER: (void)nslcd_service_bynumber(fp,session); break;
     case NSLCD_ACTION_SERVICE_ALL:      (void)nslcd_service_all(fp,session); break;
-    case NSLCD_ACTION_SHADOW_BYNAME:    (void)nslcd_shadow_byname(fp,session); break;
-    case NSLCD_ACTION_SHADOW_ALL:       (void)nslcd_shadow_all(fp,session); break;
+    case NSLCD_ACTION_SHADOW_BYNAME:    if (uid==0) (void)nslcd_shadow_byname(fp,session); break;
+    case NSLCD_ACTION_SHADOW_ALL:       if (uid==0) (void)nslcd_shadow_all(fp,session); break;
     default:
       log_log(LOG_WARNING,"invalid request id: %d",(int)action);
       break;
   }
-
   /* we're done with the request */
   myldap_session_cleanup(session);
   (void)tio_close(fp);
@@ -433,7 +413,6 @@ static void acceptconnection(MYLDAP_SESSION *session)
   int j;
   struct sockaddr_storage addr;
   socklen_t alen;
-
   /* accept a new connection */
   alen=(socklen_t)sizeof(struct sockaddr_storage);
   csock=accept(nslcd_serversocket,(struct sockaddr *)&addr,&alen);
@@ -441,13 +420,12 @@ static void acceptconnection(MYLDAP_SESSION *session)
   {
     if ((errno==EINTR)||(errno==EAGAIN)||(errno==EWOULDBLOCK))
     {
-      log_log(LOG_DEBUG,"debug: accept() failed (ignored): %s",strerror(errno));
+      log_log(LOG_DEBUG,"accept() failed (ignored): %s",strerror(errno));
       return;
     }
     log_log(LOG_ERR,"accept() failed: %s",strerror(errno));
     return;
   }
-
   /* make sure O_NONBLOCK is not inherited */
   if ((j=fcntl(csock,F_GETFL,0))<0)
   {
@@ -463,14 +441,12 @@ static void acceptconnection(MYLDAP_SESSION *session)
       log_log(LOG_WARNING,"problem closing socket: %s",strerror(errno));
     return;
   }
-
   /* handle the connection */
   handleconnection(csock,session);
 }
 
-
 /* write the current process id to the specified file */
-static void write_pidfile(const char *filename)
+static void create_pidfile(const char *filename)
 {
   FILE *fp;
   if (filename!=NULL)
@@ -493,7 +469,6 @@ static void write_pidfile(const char *filename)
   }
 }
 
-
 /* try to install signal handler and check result */
 static void install_sighandler(int signum,RETSIGTYPE (*handler) (int))
 {
@@ -509,141 +484,101 @@ static void install_sighandler(int signum,RETSIGTYPE (*handler) (int))
   }
 }
 
+static void worker_cleanup(void *arg)
+{
+  MYLDAP_SESSION *session=(MYLDAP_SESSION *)arg;
+  myldap_session_close(session);
+}
+
 static void *worker(void UNUSED(*arg))
 {
   MYLDAP_SESSION *session;
   /* create a new LDAP session */
   session=myldap_create_session();
+  /* clean up the session if we're done */
+  pthread_cleanup_push(worker_cleanup,session);
   /* start waiting for incoming connections */
-  while (nslcd_exitsignal==0)
+  while (1)
   {
     /* wait for a new connection */
     acceptconnection(session);
   }
+  pthread_cleanup_pop(1);
   return NULL;
 }
 
 /* the main program... */
 int main(int argc,char *argv[])
 {
-  gid_t mygid=(gid_t)-1;
-  uid_t myuid=(uid_t)-1;
   int i;
-
   /* parse the command line */
   parse_cmdline(argc,argv);
-
   /* clear the environment */
   /* TODO:implement */
-
   /* check if we are already running */
   /* FIXME: implement (maybe pass along options or commands) */
-
   /* disable ldap lookups of host names to avoid lookup loop
      and fall back to files dns (a sensible default) */
   /* TODO: parse /etc/nsswitch ourselves and just remove ldap from the list */
+#ifdef HAVE___NSS_CONFIGURE_LOOKUP
   if (__nss_configure_lookup("hosts","files dns"))
     log_log(LOG_ERR,"unable to override hosts lookup method: %s",strerror(errno));
-
+#endif /* HAVE___NSS_CONFIGURE_LOOKUP */
+  /* FIXME: have some other mechanism for systems that don't have this */
   /* read configuration file */
   cfg_init(NSS_LDAP_PATH_CONF);
   nslcd_cfg->ldc_debug=nslcd_debugging?nslcd_debugging-1:0;
-
   /* daemonize */
   if ((!nslcd_debugging)&&(daemon(0,0)<0))
   {
     log_log(LOG_ERR,"unable to daemonize: %s",strerror(errno));
     exit(EXIT_FAILURE);
   }
-
   /* set default mode for pidfile and socket */
   (void)umask((mode_t)0022);
-
   /* intilialize logging */
   if (!nslcd_debugging)
     log_startlogging();
   log_log(LOG_INFO,"version %s starting",VERSION);
-
   /* install handler to close stuff off on exit and log notice */
   if (atexit(exithandler))
   {
     log_log(LOG_ERR,"atexit() failed: %s",strerror(errno));
     exit(EXIT_FAILURE);
   }
-
   /* write pidfile */
-  write_pidfile(NSLCD_PIDFILE);
-
+  create_pidfile(NSLCD_PIDFILE);
   /* create socket */
-  nslcd_serversocket=open_socket();
-
+  nslcd_serversocket=create_socket();
 #ifdef HAVE_SETGROUPS
   /* drop all supplemental groups */
   if (setgroups(0,NULL)<0)
-  {
     log_log(LOG_WARNING,"cannot setgroups(0,NULL) (ignored): %s",strerror(errno));
-  }
   else
-  {
-    log_log(LOG_DEBUG,"debug: setgroups(0,NULL) done");
-  }
+    log_log(LOG_DEBUG,"setgroups(0,NULL) done");
 #else /* HAVE_SETGROUPS */
-  log_log(LOG_DEBUG,"debug: setgroups() not available");
+  log_log(LOG_DEBUG,"setgroups() not available");
 #endif /* not HAVE_SETGROUPS */
-
-#ifdef USE_CAPABILITIES
-  /* if we are using capbilities, set them to be kept
-     across setuid() calls so we can limit them later on */
-  if (prctl(PR_SET_KEEPCAPS,1))
-  {
-    log_log(LOG_ERR,"cannot prctl(PR_SET_KEEPCAPS,1): %s",strerror(errno));
-    exit(EXIT_FAILURE);
-  }
-  log_log(LOG_DEBUG,"debug: prctl(PR_SET_KEEPCAPS,1) done");
-  /* dump the current capabilities */
-  caps=cap_get_proc();
-  log_log(LOG_DEBUG,"debug: current capabilities: %s",cap_to_text(caps,NULL));
-  cap_free(caps);
-#endif /* USE_CAPABILITIES */
-
   /* change to nslcd gid */
-  if (mygid!=((gid_t)-1))
+  if (nslcd_cfg->ldc_gid!=NOGID)
   {
-    if (setgid(mygid)!=0)
+    if (setgid(nslcd_cfg->ldc_gid)!=0)
     {
-      log_log(LOG_ERR,"cannot setgid(%d): %s",(int)mygid,strerror(errno));
+      log_log(LOG_ERR,"cannot setgid(%d): %s",(int)nslcd_cfg->ldc_gid,strerror(errno));
       exit(EXIT_FAILURE);
     }
-    log_log(LOG_DEBUG,"debug: setgid(%d) done",mygid);
+    log_log(LOG_DEBUG,"setgid(%d) done",(int)nslcd_cfg->ldc_gid);
   }
-
   /* change to nslcd uid */
-  if (myuid!=((uid_t)-1))
+  if (nslcd_cfg->ldc_uid!=NOUID)
   {
-    if (setuid(myuid)!=0)
+    if (setuid(nslcd_cfg->ldc_uid)!=0)
     {
-      log_log(LOG_ERR,"cannot setuid(%d): %s",(int)myuid,strerror(errno));
+      log_log(LOG_ERR,"cannot setuid(%d): %s",(int)nslcd_cfg->ldc_uid,strerror(errno));
       exit(EXIT_FAILURE);
     }
-    log_log(LOG_DEBUG,"debug: setuid(%d) done",myuid);
+    log_log(LOG_DEBUG,"setuid(%d) done",(int)nslcd_cfg->ldc_uid);
   }
-
-#ifdef USE_CAPABILITIES
-  /* limit the capabilities */
-  if (cap_set_proc(mycapabilities)!=0)
-  {
-    log_log(LOG_ERR,"cannot cap_set_proc(%s): %s",cap_to_text(mycapabilities,NULL),strerror(errno));
-    exit(EXIT_FAILURE);
-  }
-  log_log(LOG_DEBUG,"debug: cap_set_proc(%2) done",cap_to_text(mycapabilities,NULL));
-  /* we no longer need this so we should free it */
-  cap_free(mycapabilities);
-  /* dump the current capabilities */
-  caps=cap_get_proc();
-  log_log(LOG_DEBUG,"debug: current capabilities: %s",cap_to_text(caps,NULL));
-  cap_free(caps);
-#endif /* USE_CAPABILITIES */
-
   /* install signalhandlers for some signals */
   install_sighandler(SIGHUP, sigexit_handler);
   install_sighandler(SIGINT, sigexit_handler);
@@ -654,11 +589,15 @@ int main(int argc,char *argv[])
   install_sighandler(SIGUSR1,sigexit_handler);
   install_sighandler(SIGUSR2,sigexit_handler);
   /* TODO: install signal handlers for reloading configuration */
-
   log_log(LOG_INFO,"accepting connections");
-
   /* start worker threads */
-  for (i=0;i<NUM_THREADS;i++)
+  nslcd_threads=(pthread_t *)malloc(nslcd_cfg->ldc_threads*sizeof(pthread_t));
+  if (nslcd_threads==NULL)
+  {
+    log_log(LOG_CRIT,"main(): malloc() failed to allocate memory");
+    exit(EXIT_FAILURE);
+  }
+  for (i=0;i<nslcd_cfg->ldc_threads;i++)
   {
     if (pthread_create(&nslcd_threads[i],NULL,worker,NULL))
     {
@@ -666,7 +605,6 @@ int main(int argc,char *argv[])
       exit(EXIT_FAILURE);
     }
   }
-
   /* wait for all threads to die */
   /* BUG: this causes problems if for some reason we want to exit but one
           of our threads hangs (e.g. has one of the LDAP locks)
@@ -674,7 +612,7 @@ int main(int argc,char *argv[])
      to do general house keeping things (e.g. checking signals etc) */
   /* it is also better to always do thread_cancel() here instead of in the signal
      handler */
-  for (i=0;i<NUM_THREADS;i++)
+  for (i=0;i<nslcd_cfg->ldc_threads;i++)
   {
     if (pthread_join(nslcd_threads[i],NULL))
     {
@@ -682,13 +620,12 @@ int main(int argc,char *argv[])
       exit(EXIT_FAILURE);
     }
   }
-
+  free(nslcd_threads);
   /* print something about received signals */
   if (nslcd_exitsignal!=0)
   {
     log_log(LOG_INFO,"caught signal %s (%d), shutting down",
                  signame(nslcd_exitsignal),nslcd_exitsignal);
   }
-
   return EXIT_FAILURE;
 }
